#![feature(future_join, box_as_ptr, let_chains)]

extern crate async_std;
extern crate juno_build;
extern crate rand;

use core::ptr::copy_nonoverlapping;

use rand::random;

juno_build::juno!("matmul");

fn main() {
    async_std::task::block_on(async {
        const I: usize = 256;
        const J: usize = 64;
        const K: usize = 128;
        let a: Box<[i32]> = (0..I * J).map(|_| random::<i32>() % 100).collect();
        let b: Box<[i32]> = (0..J * K).map(|_| random::<i32>() % 100).collect();
        let mut a_bytes: Box<[u8]> = Box::new([0; I * J * 4]);
        let mut b_bytes: Box<[u8]> = Box::new([0; J * K * 4]);
        unsafe {
            copy_nonoverlapping(
                Box::as_ptr(&a) as *const u8,
                Box::as_mut_ptr(&mut a_bytes) as *mut u8,
                I * J * 4,
            );
            copy_nonoverlapping(
                Box::as_ptr(&b) as *const u8,
                Box::as_mut_ptr(&mut b_bytes) as *mut u8,
                J * K * 4,
            );
        };
        let c_bytes = matmul(I as u64, J as u64, K as u64, a_bytes, b_bytes).await;
        let mut c: Box<[i32]> = (0..I * K).map(|_| 0).collect();
        unsafe {
            copy_nonoverlapping(
                Box::as_ptr(&c_bytes) as *const u8,
                Box::as_mut_ptr(&mut c) as *mut u8,
                I * K * 4,
            );
        };
        let mut correct_c: Box<[i32]> = (0..I * K).map(|_| 0).collect();
        for i in 0..I {
            for k in 0..K {
                for j in 0..J {
                    correct_c[i * K + k] += a[i * J + j] * b[j * K + k];
                }
            }
        }
        assert_eq!(c, correct_c);
    });
}

#[test]
fn matmul_test() {
    main();
}
